#include "markovChain.h"
#include "felsenstein81.h"
#include "homogeneous1DRandomWalk.h"
#include "Distribution/discreteDistribution.h"
#include "transitionStructure/eventMixture.h"
#include <iostream>
#include <algorithm>

// The application code devised for testing, AJM's version!
//

int main( int argc, char** argv ) 
{
  double pro[4] = { 0.1, 0.2, 0.3, 0.4 };

  felsenstein81* c1 = new felsenstein81( pro, 10.0 );
  discreteDistribution* staDis1 = c1->stationaryDistribution();
  simulationResult* simRes1 = c1->simulateChain( 100000.0, true, false, false, true );
  discreteDistribution* staDis2 = simRes1->getDistribution();

  c1->makeMarkovChain();
  c1->write(stdout,false);
  // markovChain *c2 = c1->copy();
  // Distribution* staDis3 = c1->stationaryDistribution( true );
  markovChain* c2 = static_cast<markovChain*>(c1);
  Distribution* staDis3 = c2->stationaryDistribution(true);
  simulationResult* simRes2 = c2->simulateChain( 100000.0, true, false, false, true);
  Distribution* staDis4 = simRes2->getDistribution();

  FILE *out = fopen( "resultat.txt", "w" );
  staDis1->write( out, -4 );
  fprintf( out, "\n" );
  staDis2->write( out, -4 );
  fprintf( out, "\n" );
  staDis3->write( out, -4 );
  fprintf( out, "\n" );
  staDis4->write( out, -4 );
  fprintf( out, "\n" );
  fprintf( out, "Distance L1 (c1,c2) = %10.6f\n", 
	   staDis1->distanceL1( staDis2 ) );
  fprintf( out, "Distance L1 (c2,c1) = %10.6f\n", 
	   staDis2->distanceL1( staDis1 ) );
  //// Deux lignes ci-dessous a de-commenter pour tester
  fprintf( out, "Distance L1 (c1,c3) = %10.6f\n", 
  	   staDis1->distanceL1( staDis3 ) );
  fprintf( out, "Distance L1 (c2,c3) = %10.6f\n", 
	   staDis2->distanceL1( staDis3 ) );
  fprintf( out, "Distance L1 (c4,c1) = %10.6f\n", 
	   staDis4->distanceL1( staDis1 ) );

  // essais avec DTMC

  homogeneous1DRandomWalk* c3 = new homogeneous1DRandomWalk( 10, 0.4, 0.3 );
  // homogeneous1DRandomWalk* c3 = new homogeneous1DRandomWalk( 10, 0.5, 0.5 );

  Distribution* staDis5 = c3->stationaryDistribution( 9 );
  staDis5->write( out, -4 );
  fprintf( out, "\n" );
  c3->write( "Xborne", "rw1d" );
  markovChain* c4 = new markovChain( "Xborne", NULL, 0, "rw1d", true );
  Distribution* staDis6 = c4->stationaryDistributionGthLD();
  staDis6->write( out, -4 );
  fprintf( out, "\n" );
  Distribution* staDis7 = c4->stationaryDistributionSOR();
  staDis7->write( out, -4 );
  fprintf( out, "\n" );
  c3->write( "MARCA", "rw1d" );
  c3->write( "PSI3", "rw1d" );
  markovChain* c5 = new markovChain( "PSI", NULL, 0, "rw1d", true );
  simulationResult* simRes3 = c5->simulatePSI( 100000, true, true, false );
  Distribution* staDis8 = simRes3->getDistribution();
  if ( NULL != staDis8 ) {
    fprintf( out, "Distrib simulation forward PSI\n" );
    staDis8->write( out, -4 );
    fprintf( out, "\n" );
  }
  simulationResult* simRes4 = c5->stationaryDistributionSample(10000);
  Distribution* staDis9 = simRes4->getDistribution();
  if ( NULL != staDis9 ) {
    fprintf( out, "Distrib simulation backward PSI\n" );
    staDis9->write( out, -4 );
    fprintf( out, "\n" );
  }


  bool setI[10] = { false };
  setI[0] = true;
  c3->makeMarkovChain();
  double* tau = c3->averageHittingTime(setI);

  for ( int i = 0; i < 10; i++ ) {
    fprintf( stdout, "Avg hit time of 0 from %2d: %12.6f\n", i, tau[i] );
  }

  sparseMatrix* gen = dynamic_cast<sparseMatrix*>(c3->generator());
  eventMixture* em = new eventMixture( gen );
  // em->write( stdout, "Ers" );

  // cleanup
  fclose( out );
  delete staDis1;
  // delete staDis2; // NO! staDis2 is the distribution inside a simulationResult
  delete staDis3;
  // delete staDis4; // NO! same as for staDis2
  delete staDis5;
  delete staDis6;
  delete staDis7;
  // delete staDis8; // NO! same as for staDis2
  delete c1;
  // delete c2; // NO! c2 was not created with "new" but with a cast
  delete c3;
  delete c4;
  delete c5;
  delete simRes1;
  delete simRes2;
  delete simRes3;

  free( tau );

  return 0;
}
