#include "Set/marmoteBox.h"
#include "transitionStructure/sparseMatrix.h"
#include "markovChain.h"
#include <stdlib.h>

void doExperiment( int n1, int n2, double p[2], double q[2] )
{
  int sz[4] = { n1, n1, n2, n2 };
  double r = 1.0 - p[0] - p[1] - q[0] - q[1];
  bool trace = false;

  marmoteBox* stSp = new marmoteBox( 4, sz );

  int card = stSp->cardinal();

  int bf[4];
  stSp->firstState(bf);

  bool* hitSet = (bool*)malloc( card*sizeof(bool) );

  // define hitting set
  for ( int i = 0; i < card; i++ ) {
    hitSet[i] = ( bf[0] == bf[1] ) || ( bf[2] == bf[3] );
    stSp->nextState(bf);
  }

  if ( trace ) {
  // visualize hitting set
  for ( int i = 0; i < card; i++ ) {
    if ( hitSet[i] ) {
      fprintf( stdout, "%5d: ", stSp->index(bf) );
      stSp->printState(stdout,bf);
      fprintf( stdout, "\n" );
    }
    stSp->nextState(bf);
  }
  }

  sparseMatrix* myTran = new sparseMatrix( card );
  myTran->setType(DISCRETE);

  int origState[4];
  int destState[4];
  stSp->firstState(origState);

#define MAX(x,y) ( (x)>(y)?(x):(y) )
#define MIN(x,y) ( (x)<(y)?(x):(y) )

  for ( int i = 0; i < card; i++ ) {
    // move x up
    destState[0] = MIN( origState[0] + 1, sz[0]-1 );
    destState[1] = MIN( origState[1] + 1, sz[1]-1 );
    destState[2] = origState[2];
    destState[3] = origState[3];
    int j = stSp->index( destState );
    myTran->setEntry(i,j,p[0]);

    // move x down
    destState[0] = MAX( origState[0] - 1, 0 );
    destState[1] = MAX( origState[1] - 1, 0 );
    destState[2] = origState[2];
    destState[3] = origState[3];
    j = stSp->index( destState );
    myTran->setEntry(i,j,q[0]);

    // move y up
    destState[0] = origState[0];
    destState[1] = origState[1];
    destState[2] = MIN( origState[2] + 1, sz[2]-1 );
    destState[3] = MIN( origState[3] + 1, sz[2]-1 );
    j = stSp->index( destState );
    myTran->setEntry(i,j,p[1]);

    // move y down
    destState[0] = origState[0];
    destState[1] = origState[1];
    destState[2] = MAX( origState[2] - 1, 0 );
    destState[3] = MAX( origState[3] - 1, 0 );
    j = stSp->index( destState );
    myTran->setEntry(i,j,q[1]);

    // stay
    myTran->setEntry(i,i,r);

    stSp->nextState(origState);
  }

  if ( trace ) {
    myTran->write(stdout,"Ers");
  }

  markovChain* mc = new markovChain( myTran );
  mc->write( "Ers", "coupling" );

  // double* tau = mc->averageHittingTime( hitSet );
  double* otau = mc->averageHittingTimeDT_iterative( hitSet );

  int argmx = 0;
  for ( int i = 0; i < mc->stateSpaceSize(); i++ ) {
    if ( trace ) {
      // fprintf( stdout, "Avg hit time of 0 from %2d: %12.6f %12.6f\n", i, tau[i], otau[i] );
      fprintf( stdout, "Avg hit time of 0 from %2d: %12.6f\n", i, otau[i] );
    }
    if ( otau[i] > otau[argmx] ) argmx = i;
  }
  fprintf( stdout, "Max en" );
  stSp->decodeState(argmx,bf);
  stSp->printState( stdout, bf );
  fprintf( stdout, ": coupling time avg = %f\n", otau[argmx] );
  fprintf( stdout, "DATA: %3d %3d %4d %6d %12f\n", sz[0], sz[2], sz[0]+sz[2], card, otau[argmx] );

  // cleanup
  free( hitSet );
  // free( tau );
  free( otau );

  delete stSp;
  // delete myTran; // NO! will be deleted by delete mc
  delete mc;

}

int main()
{
# // symmetric values
  double p[2] = { 0.2, 0.2 };
  double q[2] = { 0.2, 0.2 };
  // asymmetric values
  // double p[2] = { 0.2, 0.1 };
  // double q[2] = { 0.25, 0.3 };
  double r = 1.0 - p[0] - p[1] - q[0] - q[1];

  fprintf( stdout, "DATA: # Coupling experiment in 2D grids of different sizes.\n" );
  fprintf( stdout, "DATA: # Probas up/down in dim 1: %5.3f/%5.3f\n", p[0], q[0] );
  fprintf( stdout, "DATA: # Probas up/down in dim 2: %5.3f/%5.3f\n", p[1], q[1] );
  fprintf( stdout, "DATA: # Probas self loop: %5.3f\n", r );
  fprintf( stdout, "DATA: #\n");
  fprintf( stdout, "DATA: # Column 1: size in dimension 1\n");
  fprintf( stdout, "DATA: # Column 2: size in dimension 2\n");
  fprintf( stdout, "DATA: # Column 3: sum of sizes\n");
  fprintf( stdout, "DATA: # Column 4: product of sizes\n");
  fprintf( stdout, "DATA: # Column 5: average coupling time\n");
  fprintf( stdout, "DATA: #\n");

  for ( int i = 2; i <= 25; i++ ) {
    fprintf( stdout, "Grid size = %d x %d\n", i, 2*i );
    doExperiment( i, 2*i, p, q );
  }

  return 0;
}
