#include "markovChain.h"
#include "homogeneousMultiDRandomWalk.h"
#include "grid2d.h"
#include "Distribution/discreteDistribution.h"

#include <iostream>
#include <algorithm>

// The application code for testing coupling times in grids
//

int main( int argc, char** argv ) 
{
  int dims[] = { 10, 10 };
  double proR[] = { 0.1, 0.25 };
  double proL[] = { 0.25, 0.2 };
  int size = dims[0]*dims[1];

  grid2d* ch = new grid2d( dims, proR, proL );

  ch->write( "Maple", "G2" );
  ch->write( "XBORNE", "G2" );

  Distribution* sDis = ch->stationaryDistribution(true);
  sDis->write( stdout, DEFAULT_PRINT_MODE );
  fprintf( stdout, "\n");

  bool* setI = (bool*)malloc( size * sizeof(bool) );
  for ( int i = 0; i < size; i++ ) setI[i] = false;
  setI[0] = true;

  double* tau = ch->averageHittingTime(setI);
  double* otau = ch->averageHittingTimeDT_iterative( setI );

  for ( int i = 0; i < size; i++ ) {
    fprintf( stdout, "Avg hit time of 0 from %2d: %12.6f %12.6f\n", i, tau[i], otau[i] );
  }

  /*
  int nbS = 1000000;
  int* samples = ch->simulateHittingTime( 15, setI, nbS );
  fprintf( stdout, "Samples:\n");
  int tot = 0;
  long int tot2 = 0;
  for ( int i = 0; i < nbS; i++ ) {
    // fprintf( stdout, "%d: %d\n", i, samples[i]);
    tot += samples[i];
    tot2 += samples[i]*samples[i];
  }
  double avg = tot/(double)nbS;
  double var = ( tot2 - nbS*avg*avg )/(nbS-1);
  fprintf( stdout, "Average: %10f, std dev: %10f\n", avg, sqrt(var) );
  */

  // check evaluate value
  double* one = (double*)malloc( size * sizeof(double));
  for ( int i = 0; i < size; i++ ) one[i] = 1.0;
  double* two = (double*)malloc( size * sizeof(double));
  ch->generator()->evaluateValue(one,two);
  fprintf( stdout, "Action of generator on one:\n" );
  for ( int i = 0; i < size; i++ ) fprintf( stdout, "%8.4f ", two[i] );
  fprintf( stdout, "\n" );

  // pareil avec le nouveau "multiDim"
  homogeneousMultiDRandomWalk* ch2 = new homogeneousMultiDRandomWalk(2, dims, proR, proL );

  discreteDistribution* dStat = ch2->stationaryDistribution();
  dStat->write( stdout, NORMAL_PRINT_MODE );
  fprintf( stdout, "\n" );
  dStat->write( stdout, MAPLE_PRINT_MODE );

  // encore plus fort
  int d4[] = { 100, 100, 100, 100 };
  double p4[] = { 0.15, 0.1, 0.05, 0.2 };
  double q4[] = { 0.05, 0.06, 0.07, 0.08 };
  bool* h4 = (bool*)malloc( 100000000*sizeof(bool) );
  if ( (bool*)NULL != h4 ) {
    for ( int i = 0; i < 99999999; i++ ) h4[i] = false;
    h4[99999999] = true;
  }
  homogeneousMultiDRandomWalk* ch3 = new homogeneousMultiDRandomWalk( 4, d4, p4, q4 );
  int* tau3 = ch3->simulateHittingTime( 0, h4, 10, 1000000 );
  for ( int i = 0; i < 10; i++ ) {
    fprintf( stdout, "Ch3: Hit time run %3d: %6d\n", i, tau3[i] );
  }

  // cleanup
  // fclose( out );
  delete ch;

  free( setI );
  free( tau );
  free( otau );
  // free( samples );
  free( one );
  free( two );
  delete sDis;

  return 0;
}
