#ifndef GRID2D_H
#define GRID2D_H

#include "markovChain.h"
#include "Distribution/discreteDistribution.h"

/**
 * @brief The 2-dimensional random walk with homogeneous transition probabilities.
 * This model is characterized by:
 * - the size N, possibly INFINITE_STATE_SPACE_SIZE: number of states
 * - two probability to jump to the right, p
 * - the probability to jump to the left, q
 * with p + q <= 1. The probability to stay at the same position is 1 - p - q.
*/

class grid2d : public markovChain 
{
protected:
  int _nbDims; /**< @brief number of dimensions of the grid. Is equal to 2 here */
  double* _stateSpace;	/**< @brief representation of the state space when finite */
  int *_dimSize;          /**< @brief size of the state space in each dimension */
  double* _p;            /**< @brief the probability to jump to the right */
  double* _q;            /**< @brief the probability to jump to the left */
  double _r; 		/**< @brief r = 1 - sum_i( p_i + q_i ) is the proba of staying in the same state */

private:
  int *_mu; /**< @brief array for state space offsets */

public:
  grid2d( double* l, double* m );
  grid2d( int* sz, double* l, double* m );
  ~grid2d();

  void makeMarkovChain();

public:
  int* simulateHittingTime(int iState, bool *hittingSet, int nbSamples , int tMax);

public:
  void write(string format, string prefix);

};

#endif // GRID2D_H
