#ifndef LAYEREDSTATESPACE_H
#define LAYEREDSTATESPACE_H

#include "Set/marmoteSet.h"

class layeredStateSpace : public marmoteSet
{
private:
  int _N; /**< data buffer capacity */
  int _E1; /**< DEQ (dominant energy queue) capacity */
  int _E2; /**< SEQ (secondary energy queue) capacity */
  int _M; /**< number of states of the environment process */
  int _nu; /**< number of phases of the service process */

private:
  int _S;
  int _L0;
  int _L;

public:
  /**
   * @brief Constructor for a state space.
   * @param N data buffer capacity
   * @param E1 DEQ (dominant energy queue) capacity
   * @param E2 SEQ (secondary energy queue) capacity
   * @param M number of states of the environment process
   * @param nu number of phases of the service process
   */
  layeredStateSpace(int N, int E1, int E2, int M, int nu );
  /**
   * @brief Destructor.
   */
  ~layeredStateSpace();
  /**
   * @brief Test whether the set is finite. Always true.
   */
  bool isFinite() { return true; }
  /**
   * @brief Tests that a state, given by its vector representation, is zero.
   * @param buffer the state to be tested
   * @return true if the state is 0, false otherwise.
   */
  bool isZero(int* buffer);
  /**
   * @brief Initializes some state buffer with the first state of the set.
   * @param buffer the buffer to be set.
   */
  void firstState(int* buffer);
  /**
   * @copydoc marmoteSet::nextState()
   */
  void nextState(int *buffer);
  /**
   * @brief utility to convert a state index into a state array
   * @author Alain Jean-Marie
   * @param index the state index
   * @param buf the state buffer to be filled
   */
  void decodeState(int index, int* buf);
  /**
   * @brief Utility to find the number of some state.
   * @param buf a state buffer
   * @return the index of the state buffer
   */
  int index(int* buf);
  /**
   * @brief Procedure for printing out a state.
   * @param out file descriptor of the stream to be used
   * @param buffer the state buffer to be printed
   */
  void printState(FILE* out, int *buffer);

  void enumerate();
};

#endif // LAYEREDSTATESPACE_H
