
discreteDistribution* solveProductForm( problemData *pbm, layeredStateSpace *sp )
{
  discreteDistribution* res;

  double q2 = pbm->lambda_e2 / ( pbm->u2 + pbm->lambda_minus );
  double q1 = pbm->lambda_p / ( pbm->lambda_e1 + pbm->lambda_minus * q2 );
  double q1tilde = ( pbm->lambda_e1 + pbm->lambda_minus * q2 ) / ( pbm->lambda_p + pbm->u1 );
  double C = ( 1.0 - q1 )*( 1.0 - q1tilde )*( 1.0 - q2 ) / ( 1.0 - q1*q1tilde );

  fprintf( stderr, "# productForm: q1  = %e\n", q1 );
  fprintf( stderr, "# productForm: q1~ = %e\n", q1tilde );
  fprintf( stderr, "# productForm: q2  = %e\n", q2 );

  double* probas = (double*) calloc( sp->cardinal(), sizeof(double) );
  double* values = (double*) calloc( sp->cardinal(), sizeof(double) );

  int stateBuffer[5];
  sp->firstState(stateBuffer);

  int idx = 0;
  do {
    values[idx] = idx;

    probas[idx] = C*pow( q2, stateBuffer[3] );
    if ( stateBuffer[0] > 0 ) {
      probas[idx] *= pow( q1, stateBuffer[0] );
    }
    else if ( stateBuffer[2] > 0) {
      probas[idx] *= pow( q1tilde, stateBuffer[2] );
    }

    idx++;
    sp->nextState(stateBuffer);

  } while (!sp->isZero(stateBuffer));

  res = new discreteDistribution( sp->cardinal(), values, probas );

  res->write( stdout, 1 );
  free( values );
  free( probas );

  return res;
}
